import java.io.*;
import java.math.*;
import java.util.*;

public class TestGen {
    int testCount, subtaskCount;
    PrintWriter testsDesc;

    Random random = new Random(787788);

    final int MAX_N = 200_000;

    String[][] samples = new String[][]{{"123456", "326"}, {"123456", "12345"}};  
    String[][] hand = new String[][]{{"0", "0"}, {"00", "0"}, {"0", "00"}, {"550", "00"}, {"505", "00"}};

    void write(String[] a, String folder, String subtask, int points, String comment) throws IOException {
        String path = String.format("../%s/%02d", folder, ++testCount);
        PrintWriter out = new PrintWriter(path);
        for (String s : a) {
            out.println(s);
        }
        out.close();
        testsDesc.println(String.format("%s, folder=%s, subtask=%s, points=%d, comment=%s", path, folder, subtask, points, comment));
        System.out.println(String.format("Test %d is generated", testCount));
    }

    void genPreliminary() throws IOException {
        for (int i = 0; i < samples.length; i++) {
            write(samples[i], "preliminary", "subtask0", 0, String.format("\"sample %d\"", i + 1));
        }
        testCount = 0;
    }

    void genHand() throws IOException {
        for (int i = 0; i < hand.length; i++) {
            write(hand[i], "tests", "subtask" + Integer.toString(subtaskCount), 0, String.format("\"hand %d\"", i + 1));
        }
    }

    StringBuilder[] genRandOkPair(int n, int m) {
        ArrayList<Integer> a = new ArrayList<>();

        for (int i = 0; i < n; i++) {
            a.add(random.nextInt(10));
        }

        StringBuilder s1 = new StringBuilder();
        for (int i = 0; i < n; i++) {
            s1.append((char)('0' + a.get(i)));
        }

        ArrayList<Integer> ends = new ArrayList<>();
        for (int i = 0; i < n - 1; i++) {
            ends.add(i);
        }
        for (int i = 0; i < ends.size(); i++) {
            int x = random.nextInt(i + 1);
            int o = ends.get(x);
            ends.set(x, ends.get(i));
            ends.set(i, o);
        }
        ArrayList<Integer> ends2 = new ArrayList<>();
        for (int i = 0; i < m - 1; i++) {
            ends2.add(ends.get(i));
        }
        ends = ends2;
        ends.add(n - 1);
        Collections.sort(ends);

        StringBuilder s2 = new StringBuilder();
        int cur = 0;
        int j = 0;
        for (int i = 0; i < n; i++) {
            cur = (cur + a.get(i)) % 10;
            if (ends.get(j) == i) {
                s2.append((char)(cur + '0'));
                cur = 0;
                j++;
            }
        }

        return new StringBuilder[]{s1, s2};
    }

    void genRandYes(int n, int m) throws IOException {
        StringBuilder s[] = genRandOkPair(n, m);
        write(new String[]{s[0].toString(), s[1].toString()}, "tests", "subtask" + Integer.toString(subtaskCount), 0, String.format("\"random, n=%d, m=%d\"", n, m));
    }

    void genRandNo(int n, int m) throws IOException {
        StringBuilder s[] = genRandOkPair(n, m);
        if ((random.nextBoolean() || m == 1) && m != MAX_N) {
            s[1].insert(random.nextInt(s[1].length()), '0' + random.nextInt(10));
        } else {
            s[1].deleteCharAt(random.nextInt(s[1].length()));
        }
        write(new String[]{s[0].toString(), s[1].toString()}, "tests", "subtask" + Integer.toString(subtaskCount), 0, String.format("\"random, n=%d, m=%d\"", n, m));
    }

    void genSubtask(int n, int points) throws IOException {
        ++subtaskCount;

        System.err.println("Subtask " + subtaskCount);
        if (subtaskCount == 1) {
            genHand();
        }

        for (int i = 0; i < 5; i++) {
            int n1 = random.nextInt(n / 2) + n / 2;
            int n2 = random.nextInt(n1 * 9 / 10) + n1 / 10 + 1;
            genRandYes(n1, n2);
            genRandNo(n1, n2);
        }
        for (int i = 1; i <= 5; i++) {
            genRandYes(n, n / i);
            genRandNo(n, n / i);
        }
        genRandYes(n, 1);
        genRandNo(n, 1);

        testsDesc.println(String.format("comment_subtask%d=\"n &lt; %d\"", subtaskCount, n));
        testsDesc.println(String.format("bonus_subtask%d=%d", subtaskCount, points));       
    }

    void run() throws IOException {
        new File("../preliminary").mkdir();
        new File("../tests").mkdir();
        testsDesc = new PrintWriter("../tests.desc");

        genPreliminary();
        genSubtask(10, 20);
        genSubtask(500, 30);
        genSubtask(MAX_N, 50);

        testsDesc.close();
    }

    public static void main(String[] args) throws IOException {
        new TestGen().run();
    }
}
