#include <iostream>
#include <vector>
#include <set>
#include <cmath>
#include <cassert>
#include <algorithm>
using namespace std;

#define TASK "player"

typedef long long ll;

struct Segment {
    ll l, r, sm;
    int f, length, id;
    Segment() {}
    Segment(ll l, ll r, int f, int id):l(l), r(r), f(f), id(id), length(r - l), sm(1LL * (r - l) * f) {}

    bool operator < (const Segment & other) const {
        return f > other.f || f == other.f && l < other.l;
    }

    bool operator == (const Segment & other) const {
        return f == other.f && l == other.l && r == other.r;
    }

    int len() const {
        return length;
    }

    ll sum() const {
        return sm;
    }
};

typedef pair<ll, ll> DescentInfo;


const ll INF_LL = 1e18 + 10;

struct ZbanNapishiDopZblTree {
    struct Node {
        int maxF;
        ll sumT, sumFun;
    };

    vector<Node> t;

    int acel, size;
    ZbanNapishiDopZblTree(int acel, int n):acel(acel) {
        size = 1;
        while (size < n) size *= 2;
        t.resize(2 * size);
    }

    void update(int pos, const Segment& val) {
        pos += size;
        while (pos >= 1) {
            t[pos].maxF = max(t[pos].maxF, val.f);
            t[pos].sumT += val.len();
            t[pos].sumFun = min(t[pos].sumFun + val.sum(), INF_LL);
            pos >>= 1;
        }
    }


    DescentInfo descentByV(const Segment & next, int acel) const {
        int v = 1;
        ll sumT = 0, sumFun = 0;
        ll val = 1LL * (acel - 1) * next.f;
        while (v < size) {
            if (val < 1LL * t[2 * v].maxF * acel) {
                sumT += t[2 * v].sumT;
                sumFun = min(sumFun + t[2 * v].sumFun, INF_LL);
                v = 2 * v + 1;
            } else
                v *= 2;
        }
        return DescentInfo(sumT, sumFun);
    }

    DescentInfo descentBySumFun(const Segment & next, ll f) const {
        int v = 1;
        ll val = next.sum();
        ll sumT = 0, sumFun = 0;
        while (v < size) {
            if (f - t[2 * v].sumFun > val) {
                f -= t[2 * v].sumFun;
                sumFun = min(sumFun + t[2 * v].sumFun, INF_LL);
                sumT += t[2 * v].sumT;
                v = 2 * v + 1;
            } else
                v *= 2;
        }
        return DescentInfo(sumT, sumFun);
    }
};

void relax(const ZbanNapishiDopZblTree& tree, double& ans, int acel, ll fun, const Segment& next) {
    DescentInfo bound = tree.descentByV(next, acel);
    if (fun < bound.second)
        return;
    if (fun - bound.second > next.sum())
        bound = tree.descentBySumFun(next, fun);
    ans = min(ans, 1.0 * (next.l - bound.first) / acel + bound.first + 1.0 * (fun - bound.second) / next.f);
}

int main() {
    freopen(TASK".in", "r", stdin);
    freopen(TASK".out", "w", stdout);

    int n, v, ff;
    scanf("%d%d%d", &n, &v, &ff);
    ll f = ff;

    vector<Segment> a;
    ll sumT = 0;
    int id = 0;
    for (int i = 0; i < n; ++i) {
        int t, k;
        scanf("%d%d", &t, &k);
        for (int j = 0; j < k; ++j) {
            int l, r, fij;
            scanf("%d%d%d", &l, &r, &fij);
            a.push_back(Segment(l + sumT, r + sumT, fij, id++));
        }
        sumT += t;
    }
    Segment last = a.back();
    a.push_back(Segment(last.r, last.r, 0, id++));

    vector<int> pos(id);
    vector<Segment> b = a;
    sort(b.begin(), b.end());
    for (int i = 0; i < b.size(); ++i)
        pos[b[i].id] = i;

    ZbanNapishiDopZblTree tree(v, id);
    ll sf = f;

    int i = 0;
    for (; i < a.size() && f - a[i].sum() > 0; ++i) {
        f -= a[i].sum();
        tree.update(pos[a[i].id], a[i]);
    }

    if (i == a.size()) {
        printf("-1\n");
        return 0;
    }

    double ans = 1e15;
    for (; i + 1 < a.size(); ++i) {
        relax(tree, ans, v, sf, a[i]);
        tree.update(pos[a[i].id], a[i]);
    }

    printf("%.10f\n", ans);
    return 0;
}
