
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

public class Validate {
    public static final int MAX_N = 1000_00;
    public static final int MAX_VAL = 10000;
    public static final int MAX_K = 17;

    public void run() {
        StrictScanner inf = new StrictScanner(System.in);
        int n = inf.nextInt();
        int k = inf.nextInt();
        inf.nextLine();
        ensureLimits(n, 1, MAX_N, "N doesn't satisfy to restrictions");
        ensureLimits(k, 1, MAX_K, "K doesn't satisfy to restrictions");
        String a = inf.next();
        inf.nextLine();
        String b = inf.next();
        inf.nextLine();
        ensure(a.length() == n && b.length() == n, "Length of string doesn't equals n");
        checkBinary(a);
        checkBinary(b);
        int[] h = new int[k];
        int[] p = new int[k];
        boolean[] was = new boolean[17];

        for (int i = 0; i < k; ++i) {
            h[i] = inf.nextInt();
            p[i] = inf.nextInt();
            inf.nextLine();
            ensureLimits(h[i], 0, 16, String.format("0 <= h[%d] <= 16 isn't satisfied", i));
            ensure((1 << h[i]) <= n, String.format("2^h[%d] doesn't satisfy to restrictions", i));
            ensureLimits(p[i], 1, MAX_VAL, String.format("p[%d] doesn't satisfy to restrictions", i));
            if (was[h[i]])
                throw new AssertionError(h[i] + " is repeated");
            was[h[i]] = true;
        }

        int m = inf.nextInt();
        inf.nextLine();
        ensureLimits(m, 1, MAX_N, "M doesn't satisfy to restrictions");

        for (int i = 1; i <= m; ++i) {
            int num = inf.nextInt();
            int bit = inf.nextInt();
            inf.nextLine();
            ensureLimits(num, 1, 2, String.format("num[%d] doesn't satisfy to restrictions", i));
            ensureLimits(bit, 0, n - 1, String.format("bit[%d] doesn't satisfy to restrictions", i));
        }
        inf.close();
    }

    private void checkBinary(String s) {
        for (int i = 0; i < s.length(); ++i)
            if (s.charAt(i) != '1' && s.charAt(i) != '0')
                throw new AssertionError("String isn't binary");
    }

    private boolean ensureLower(String s) {
        for (int i = 0; i < s.length(); ++i)
            if (!Character.isLowerCase(s.charAt(i)))
                return false;
        return true;
    }

    public static void main(String[] args) {
        new Validate().run();
    }

    public class StrictScanner {
        private final BufferedReader in;
        private String line = "";
        private int pos;
        private int lineNo;

        public StrictScanner(InputStream source) {
            in = new BufferedReader(new InputStreamReader(source));
            nextLine();
        }

        public void close() {
            ensure(line == null, "Extra data at the end of file");
            try {
                in.close();
            } catch (IOException e) {
                throw new AssertionError("Failed to close with " + e);
            }
        }

        public void nextLine() {
            ensure(line != null, "EOF");
            ensure(pos == line.length(), "Extra characters on line " + lineNo);
            try {
                line = in.readLine();
            } catch (IOException e) {
                throw new AssertionError("Failed to read line with " + e);
            }
            pos = 0;
            lineNo++;
        }

        public String getNextLine() {
            String st = "";
            while (!isEoln()) {
                st = st + next();
            }
            return st;
        }


        public boolean isEoln() {
            return (pos == line.length());
        }

        public String next() {
            ensure(line != null, "EOF");
            ensure(line.length() > 0, "Empty line " + lineNo);
            if (pos == 0)
                ensure(line.charAt(0) > ' ', "Line " + lineNo
                        + " starts with whitespace");
            else {
                ensure(pos < line.length(), "Line " + lineNo + " is over");
                ensure(line.charAt(pos) == ' ', "Wrong whitespace on line "
                        + lineNo);
                pos++;
                ensure(pos < line.length(), "Line " + lineNo + " is over");
                ensure(line.charAt(0) > ' ', "Line " + lineNo
                        + " has double whitespace");
            }
            StringBuilder sb = new StringBuilder();
            while (pos < line.length() && line.charAt(pos) > ' ')
                sb.append(line.charAt(pos++));
            return sb.toString();
        }

        public int nextInt() {
            String s = next();
            ensure(s.length() == 1 || s.charAt(0) != '0',
                    "Extra leading zero in number " + s + " on line " + lineNo);
            ensure(s.charAt(0) != '+', "Extra leading '+' in number " + s
                    + " on line " + lineNo);
            try {
                return Integer.parseInt(s);
            } catch (NumberFormatException e) {
                throw new AssertionError("Malformed number " + s + " on line "
                        + lineNo);
            }
        }

        public long nextLong() {
            String s = next();
            ensure(s.length() == 1 || s.charAt(0) != '0',
                    "Extra leading zero in number " + s + " on line " + lineNo);
            ensure(s.charAt(0) != '+', "Extra leading '+' in number " + s
                    + " on line " + lineNo);
            try {
                return Long.parseLong(s);
            } catch (NumberFormatException e) {
                throw new AssertionError("Malformed number " + s + " on line "
                        + lineNo);
            }
        }

        public double nextDouble() {
            String s = next();
            ensure(s.length() == 1 || s.startsWith("0.") || s.charAt(0) != '0',
                    "Extra leading zero in number " + s + " on line " + lineNo);
            ensure(s.charAt(0) != '+', "Extra leading '+' in number " + s
                    + " on line " + lineNo);
            try {
                return Double.parseDouble(s);
            } catch (NumberFormatException e) {
                throw new AssertionError("Malformed number " + s + " on line "
                        + lineNo);
            }
        }
    }

    static void ensure(boolean b, String message) {
        if (!b) {
            throw new AssertionError(message);
        }
    }

    void ensureString(String s, String alph, String text) {
        for (int i = 0; i < s.length(); i++) {
            ensure(alph.indexOf(s.charAt(i)) != -1, text);
        }
    }

    void ensureLimits(long n, long from, long to, String name) {
        ensure(from <= n && n <= to, name + " must be from " + from + " to "
                + to);
    }

    void ensureLimits(int n, int from, int to, String name) {
        ensure(from <= n && n <= to, name + " must be from " + from + " to "
                + to);
    }

    String line(int x) {
        return "Problem in line #" + Integer.toString(x);
    }
}


